@@ -4,6 +4,7 @@ from botocore.client import BaseClient
 from dstack.aws import logs, artifacts, jobs, run_names, runs, runners, tags, repos, secrets, config
 from dstack.backend import *
 from dstack.config import AwsBackendConfig
+from dstack.jobs import AppSpec
 
 
 class AwsBackend(Backend):
@@ -40,9 +41,9 @@ class AwsBackend(Backend):
                                 region_name=self.backend_config.region_name)
         return session.client("sts")
 
-    def configure(self, silent: bool):
-        config.configure(self._iam_client(), self._s3_client(), self.backend_config.bucket_name,
-                         self.backend_config.region_name, silent)
+    def configure(self, silent: bool) -> bool:
+        return config.configure(self._iam_client(), self._s3_client(), self.backend_config.bucket_name,
+                                self.backend_config.region_name, silent)
 
     def create_run(self, repo_user_name: str, repo_name: str) -> str:
         return runs.create_run(self._s3_client(), self._logs_client(), self.backend_config.bucket_name,
@@ -92,7 +93,17 @@ class AwsBackend(Backend):
                               self.backend_config.bucket_name, repo_user_name, repo_name, job_heads, start_time,
                               attached)
 
-    def list_run_artifact_files(self, repo_user_name: str, repo_name: str, run_name: str) -> List[Tuple[str, str, int]]:
+    def query_logs(self, repo_user_name: str, repo_name: str, run_name: str, start_time: int, end_time: Optional[int],
+                   next_token: Optional[str], job_host_names: Dict[str, Optional[str]],
+                   job_ports: Dict[str, Optional[List[int]]], job_app_specs: Dict[str, Optional[List[AppSpec]]]) \
+            -> Tuple[List[LogEvent], Optional[str], Dict[str, Optional[str]], Dict[str, Optional[List[int]]],
+                     Dict[str, Optional[List[AppSpec]]]]:
+        return logs.query_logs(self._s3_client(), self._logs_client(),
+                               self.backend_config.bucket_name, repo_user_name, repo_name, run_name, start_time,
+                               end_time, next_token, job_host_names, job_ports, job_app_specs)
+
+    def list_run_artifact_files(self, repo_user_name: str, repo_name: str, run_name: str) -> \
+            Generator[Tuple[str, str, int], None, None]:
         return artifacts.list_run_artifact_files(self._s3_client(), self.backend_config.bucket_name,
                                                  repo_user_name, repo_name, run_name)
 
@@ -108,9 +119,10 @@ class AwsBackend(Backend):
         return tags.get_tag_head(self._s3_client(), self.backend_config.bucket_name, repo_user_name, repo_name,
                                  tag_name)
 
-    def add_tag_from_run(self, repo_user_name: str, repo_name: str, tag_name: str, run_name: str):
+    def add_tag_from_run(self, repo_user_name: str, repo_name: str, tag_name: str, run_name: str,
+                         run_jobs: Optional[List[Job]]):
         tags.create_tag_from_run(self._s3_client(), self.backend_config.bucket_name, repo_user_name, repo_name,
-                                 tag_name, run_name)
+                                 tag_name, run_name, run_jobs)
 
     def delete_tag_head(self, repo_user_name: str, repo_name: str, tag_head: TagHead):
         tags.delete_tag(self._s3_client(), self.backend_config.bucket_name, repo_user_name, repo_name, tag_head)
