@@ -1,3 +1,4 @@
+import os
 from abc import ABC
 from pathlib import Path
 from typing import Optional
@@ -33,20 +34,26 @@ class ConfigError(Exception):
 
 
 def load_config(path: Path = get_config_path()) -> Config:
-    if path.exists():
-        with path.open() as f:
-            config_data = yaml.load(f, Loader=yaml.FullLoader)
-            backend_name = config_data.get("backend")
-            if not backend_name:
-                raise ConfigError(f"{path.resolve()} no backend is configured in {path.resolve()}")
-            elif backend_name != "aws":
-                raise ConfigError(f"Unknown backend: {backend_name}")
-            else:
-                return Config(AwsBackendConfig(config_data["bucket"],
-                                               config_data.get("region"),
-                                               config_data.get("profile")))
+    bucket_name = os.getenv("DSTACK_AWS_S3_BUCKET")
+    if bucket_name:
+        return Config(AwsBackendConfig(bucket_name,
+                                       os.getenv("DSTACK_AWS_REGION") or os.getenv("AWS_DEFAULT_REGION"),
+                                       os.getenv("DSTACK_AWS_PROFILE") or os.getenv("AWS_PROFILE")))
     else:
-        raise ConfigError(f"{path.resolve()} doesn't exist")
+        if path.exists():
+            with path.open() as f:
+                config_data = yaml.load(f, Loader=yaml.FullLoader)
+                backend_name = config_data.get("backend")
+                if not backend_name:
+                    raise ConfigError(f"{path.resolve()} no backend is configured in {path.resolve()}")
+                elif backend_name != "aws":
+                    raise ConfigError(f"Unknown backend: {backend_name}")
+                else:
+                    return Config(AwsBackendConfig(config_data["bucket"],
+                                                   config_data.get("region") or os.getenv("AWS_DEFAULT_REGION"),
+                                                   config_data.get("profile") or os.getenv("AWS_PROFILE")))
+        else:
+            raise ConfigError(f"{path.resolve()} doesn't exist")
 
 
 def write_config(config: Config, path: Path = get_config_path()):
